@@ -22,7 +22,7 @@ module XMonad.Actions.MouseGestures (
 ) where
 
 import XMonad
-import XMonad.Layout.WindowNavigation (Direction(..))
+import XMonad.Hooks.ManageDocks (Direction(..))
 
 import Data.IORef
 import qualified Data.Map as M
