@@ -20,13 +20,12 @@ module XMonad.Config.Arossato
     ) where
 
 import qualified Data.Map as M
-import System.IO (hPutStrLn)
 
 import XMonad hiding ( (|||) )
 import qualified XMonad.StackSet as W
 
 import XMonad.Actions.CycleWS
-import XMonad.Hooks.DynamicLog
+import XMonad.Hooks.DynamicLog hiding (xmobar)
 import XMonad.Hooks.ManageDocks
 import XMonad.Hooks.ServerMode
 import XMonad.Layout.Accordion
