@@ -22,6 +22,7 @@ module XMonad.Hooks.DynamicLog (
 
     -- * Drop-in loggers
     dzen,
+    xmobar,
     dynamicLog,
     dynamicLogDzen,
     dynamicLogXmobar,
@@ -51,13 +52,18 @@ module XMonad.Hooks.DynamicLog (
 import XMonad
 import Data.Maybe ( isJust, catMaybes )
 import Data.List
+import qualified Data.Map as M
 import Data.Ord ( comparing )
 import qualified XMonad.StackSet as S
 import System.IO
 import XMonad.Util.WorkspaceCompare
 import XMonad.Util.NamedWindows
 import XMonad.Util.Run
+
+import XMonad.Layout.LayoutModifier
+import XMonad.Util.Font
 import XMonad.Hooks.UrgencyHook
+import XMonad.Hooks.ManageDocks
 
 -- $usage
 -- You can use this module with the following in your @~\/.xmonad\/xmonad.hs@:
@@ -75,6 +81,9 @@ import XMonad.Hooks.UrgencyHook
 --
 -- > main = dzen $ \conf -> xmonad $ conf { <your customizations> }
 --
+-- Also you can use 'xmobar' function instead of 'dzen' in the examples above,
+-- if you have xmobar installed.
+--
 -- Alternatively, you can choose among several default status bar
 -- formats ('dynamicLog', 'dynamicLogDzen', 'dynamicLogXmobar', or
 -- 'dynamicLogXinerama') by simply setting your logHook to the
@@ -101,6 +110,8 @@ import XMonad.Hooks.UrgencyHook
 -- appropriately, either by putting it in your @.xsession@ or similar
 -- file, or by using @spawnPipe@ in your @main@ function, for example:
 --
+-- > import XMonad.Util.Run   -- for spawnPipe and hPutStrLn
+-- >
 -- > main = do
 -- >     h <- spawnPipe "xmobar -options -foo -bar"
 -- >     xmonad $ defaultConfig {
@@ -126,6 +137,8 @@ import XMonad.Hooks.UrgencyHook
 --
 --   * add an xmobarEscape function
 
+------------------------------------------------------------------------
+
 -- | Run xmonad with a dzen status bar set to some nice defaults. Output
 -- is taken from the dynamicLogWithPP hook.
 --
@@ -140,17 +153,57 @@ import XMonad.Hooks.UrgencyHook
 -- If you wish to customize the status bar format at all, you'll have to
 -- use something like 'dynamicLogWithPP' instead.
 --
-dzen :: (XConfig (Choose Tall (Choose (Mirror Tall) Full)) -> IO ()) -> IO ()
+-- The binding uses the XMonad.Hooks.ManageDocks module to automatically
+-- handle screen placement for dzen, and enables 'mod-b' for toggling
+-- the menu bar.
+--
+dzen ::
+    (XConfig
+       (ModifiedLayout AvoidStruts
+          (Choose Tall (Choose (Mirror Tall) Full))) -> IO t) -> IO t
 dzen f = do
   h <- spawnPipe ("dzen2" ++ " " ++ flags)
   f $ defaultConfig
-           { defaultGaps = [(15,0,0,0)] -- for fixed
-           , logHook = dynamicLogWithPP dzenPP
-                          { ppOutput = hPutStrLn h } }
+           { logHook   = dynamicLogWithPP dzenPP
+                          { ppOutput = hPutStrLn h }
+           ,layoutHook = avoidStrutsOn [U] (layoutHook defaultConfig)
+           ,keys       = \c -> toggleStrutsKey c `M.union` keys defaultConfig c
+           ,manageHook = manageHook defaultConfig <+> manageDocks
+           }
  where
     fg      = "'#a8a3f7'" -- n.b quoting
     bg      = "'#3f3c6d'"
-    flags   = "-e '' -w 400 -ta l -fg " ++ fg ++ " -bg " ++ bg
+    flags   = "-e 'onstart=lower' -w 400 -ta l -fg " ++ fg ++ " -bg " ++ bg
+
+
+-- | Run xmonad with a xmobar status bar set to some nice defaults. Output
+-- is taken from the dynamicLogWithPP hook.
+--
+-- > main = xmobar xmonad
+--
+-- This works pretty much the same as 'dzen' function above
+--
+xmobar ::
+    (XConfig
+       (ModifiedLayout AvoidStruts
+          (Choose Tall (Choose (Mirror Tall) Full))) -> IO t) -> IO t
+xmobar f = do
+  h <- spawnPipe "xmobar"
+  f $ defaultConfig
+           { logHook    = dynamicLogWithPP xmobarPP { ppOutput = hPutStrLn h }
+           , layoutHook = avoidStruts $ layoutHook defaultConfig
+           , keys       = \c -> toggleStrutsKey c `M.union` keys defaultConfig c
+           , manageHook = manageHook defaultConfig <+> manageDocks
+           }
+
+-- |
+-- Helper function which provides ToggleStruts keybinding
+--
+toggleStrutsKey :: XConfig t -> M.Map (KeyMask, KeySym) (X ())
+toggleStrutsKey XConfig{modMask = modm} = M.fromList
+        [ ((modm, xK_b ), sendMessage ToggleStruts) ]
+
+------------------------------------------------------------------------
 
 -- | An example log hook, which prints status information to stdout in
 -- the default format:
@@ -203,7 +256,7 @@ dynamicLogString pp = do
     -- run extra loggers, ignoring any that generate errors.
     extras <- sequence $ map (flip catchX (return Nothing)) $ ppExtras pp
 
-    return $ sepBy (ppSep pp) . ppOrder pp $
+    return $ encodeOutput . sepBy (ppSep pp) . ppOrder pp $
                         [ ws
                         , ppLayout pp ld
                         , ppTitle  pp wt
@@ -403,8 +456,8 @@ xmobarPP = defaultPP { ppCurrent = xmobarColor "yellow" "" . wrap "[" "]"
 -- example.  Note the use of 'xmobarColor' and the record update on
 -- 'defaultPP'.
 sjanssenPP :: PP
-sjanssenPP = defaultPP { ppCurrent = xmobarColor "white" "#ff000000"
-                       , ppTitle = xmobarColor "#00ee00" "" . shorten 80
+sjanssenPP = defaultPP { ppCurrent = xmobarColor "white" "black"
+                       , ppTitle = xmobarColor "#00ee00" "" . shorten 120
                        }
 
 -- | The options that byorgey likes to use with dzen, as another example.
