@@ -15,7 +15,9 @@
 module XMonad.Hooks.EwmhDesktops (
     -- * Usage
     -- $usage
+    EwmhDesktopsHook,
     ewmhDesktopsLogHook,
+    ewmhDesktopsLogHookCustom,
     ewmhDesktopsLayout
     ) where
 
@@ -37,13 +39,15 @@ import XMonad.Hooks.EventHook
 -- > import XMonad.Hooks.EwmhDesktops
 -- >
 -- > myLogHook :: X ()
--- > myLogHook = do ewmhDesktopsLogHook
--- >                return ()
+-- > myLogHook = ewmhDesktopsLogHook
 -- >
--- > layoutHook = ewmhDesktopsLayout $ avoidStruts $ simpleTabbed ||| Full ||| etc..
+-- > myLayoutHook = ewmhDesktopsLayout $ avoidStruts $ layoutHook defaultConfig
 -- >
 -- > main = xmonad defaultConfig { layoutHook = myLayouts, logHook = myLogHook }
 --
+-- 'avoidStruts' is used to automatically leave space for dock programs, and
+-- can be found in 'XMonad.Hooks.ManageDocks'.
+--
 -- For more detailed instructions on editing the layoutHook see:
 --
 -- "XMonad.Doc.Extending#The_log_hook_and_external_status_bars"
@@ -52,14 +56,23 @@ import XMonad.Hooks.EventHook
 --
 -- "XMonad.Doc.Extending#Editing_the_layout_hook"
 
+
+
+
 -- |
 -- Notifies pagers and window lists, such as those in the gnome-panel
 -- of the current state of workspaces and windows.
 ewmhDesktopsLogHook :: X ()
-ewmhDesktopsLogHook = withWindowSet $ \s -> do
+ewmhDesktopsLogHook = ewmhDesktopsLogHookCustom id
+
+
+-- |
+-- Generalized version of ewmhDesktopsLogHook that allows an arbitrary
+-- user-specified function to transform the workspace list (post-sorting)
+ewmhDesktopsLogHookCustom :: ([WindowSpace] -> [WindowSpace]) -> X ()
+ewmhDesktopsLogHookCustom f = withWindowSet $ \s -> do
     sort' <- getSortByIndex
-    let ws = sort' $ W.workspaces s
-    let wins = W.allWindows s
+    let ws = f $ sort' $ W.workspaces s
 
     setSupported
 
@@ -74,6 +87,8 @@ ewmhDesktopsLogHook = withWindowSet $ \s -> do
 
     setCurrentDesktop curr
 
+    -- all windows, with focused windows last
+    let wins =  nub . concatMap (maybe [] (\(W.Stack x l r)-> reverse l ++ r ++ [x]) . W.stack) $ ws
     setClientList wins
 
     -- Per window Desktop
@@ -100,7 +115,7 @@ ewmhDesktopsLogHook = withWindowSet $ \s -> do
 --
 --  * _NET_WM_DESKTOP (move windows to other desktops)
 --
---  * _NET_ACTIVE_WINDOW (activate another window)
+--  * _NET_ACTIVE_WINDOW (activate another window, changing workspace if needed)
 --
 ewmhDesktopsLayout :: layout a -> HandleEvent EwmhDesktopsHook layout a
 ewmhDesktopsLayout = eventHook EwmhDesktopsHook
@@ -122,6 +137,7 @@ handle ClientMessageEvent {
        a_cd <- getAtom "_NET_CURRENT_DESKTOP"
        a_d <- getAtom "_NET_WM_DESKTOP"
        a_aw <- getAtom "_NET_ACTIVE_WINDOW"
+       a_cw <- getAtom "_NET_CLOSE_WINDOW"
        if  mt == a_cd then do
                let n = fromIntegral (head d)
                if 0 <= n && n < length ws then
@@ -134,6 +150,9 @@ handle ClientMessageEvent {
                  else  trace $ "Bad _NET_DESKTOP with data[0]="++show n
         else if mt == a_aw then do
                windows $ W.focusWindow w
+        else if mt == a_cw then do
+               windows $ W.focusWindow w
+               kill
         else trace $ "Unknown ClientMessageEvent " ++ show mt
 handle _ = undefined -- does not happen, as otherwise ewmhDesktopsHook would not match
 
@@ -199,7 +218,7 @@ setSupported = withDisplay $ \dpy -> do
 
 setActiveWindow :: X ()
 setActiveWindow = withWindowSet $ \s -> withDisplay $ \dpy -> do
-    let w = fromMaybe 0 (W.peek s)
+    let w = fromMaybe none (W.peek s)
     r <- asks theRoot
     a <- getAtom "_NET_ACTIVE_WINDOW"
     c <- getAtom "WINDOW"
