@@ -4,13 +4,12 @@
 -----------------------------------------------------------------------------
 -- |
 -- Module      :  XMonad.Layout.DragPane
--- Copyright   :  (c) Spencer Janssen <sjanssen@cse.unl.edu>
+-- Copyright   :  (c) Spencer Janssen <spencerjanssen@gmail.com>
 --                    David Roundy <droundy@darcs.net>,
 --                    Andrea Rossato <andrea.rossato@unibz.it>
 -- License     :  BSD3-style (see LICENSE)
 -- 
--- Maintainer  :  David Roundy <droundy@darcs.net>
---                Andrea Rossato <andrea.rossato@unibz.it>
+-- Maintainer  :  Andrea Rossato <andrea.rossato@unibz.it>
 -- Stability   :  unstable
 -- Portability :  unportable
 --
@@ -133,4 +132,6 @@ newDragWin r = do
   let mask = Just $ exposureMask .|. buttonPressMask
   w <- createNewWindow r mask handleColor False
   showWindow  w
+  d <- asks display
+  liftIO $ lowerWindow d w
   return      w
