@@ -7,7 +7,7 @@
 -- Copyright   :  (c) 2007  David Roundy <droundy@darcs.net>
 -- License     :  BSD3-style (see LICENSE)
 --
--- Maintainer  :  David Roundy <droundy@darcs.net>
+-- Maintainer  :  Devin Mullins <me@twifkak.com>
 -- Stability   :  unstable
 -- Portability :  unportable
 --
@@ -32,6 +32,8 @@ import XMonad.Layout.LayoutModifier
 import XMonad.Util.Invisible
 import XMonad.Util.XUtils
 
+import XMonad.Hooks.ManageDocks (Direction(..))
+
 -- $usage
 -- You can use this module with the following in your @~\/.xmonad\/xmonad.hs@:
 --
@@ -67,7 +69,6 @@ data MoveWindowToWindow a = MoveWindowToWindow a a deriving ( Read, Show, Typeab
 instance Typeable a => Message (MoveWindowToWindow a)
 
 data Navigate = Go Direction | Swap Direction | Move Direction deriving ( Read, Show, Typeable )
-data Direction = U | D | R | L deriving ( Read, Show, Eq, Ord, Enum, Bounded )
 instance Message Navigate
 
 data WNConfig =
@@ -92,7 +93,7 @@ navigateBrightness f | f > 1 = navigateBrightness 1
 navigateBrightness f = defaultWNConfig { brightness = Just f }
 
 defaultWNConfig :: WNConfig
-defaultWNConfig = WNC (Just 0.5) "#0000FF" "#00FFFF" "#FF0000" "#FF00FF"
+defaultWNConfig = WNC (Just 0.4) "#0000FF" "#00FFFF" "#FF0000" "#FF00FF"
 
 data NavigationState a = NS Point [(a,Rectangle)]
 
@@ -126,10 +127,10 @@ instance LayoutModifier WindowNavigation Window where
                wrs = filter ((`elem` existing_wins) . fst) $ filter ((/=r) . snd) $
                      filter ((/=w) . fst) origwrs
                wnavigable = nub $ concatMap
-                            (\d -> truncHead $ sortby d $ filter (inr d pt . snd) wrs) [U,D,R,L]
+                            (\d -> truncHead $ navigable d pt wrs) [U,D,R,L]
                wnavigablec = nub $ concatMap
                             (\d -> map (\(win,_) -> (win,dirc d)) $
-                                   truncHead $ sortby d $ filter (inr d pt . snd) wrs) [U,D,R,L]
+                                   truncHead $ navigable d pt wrs) [U,D,R,L]
                wothers = case state of Just (NS _ wo) -> map fst wo
                                        _              -> []
            mapM_ (sc nbc) (wothers \\ map fst wnavigable)
@@ -138,7 +139,7 @@ instance LayoutModifier WindowNavigation Window where
 
     handleMessOrMaybeModifyIt (WindowNavigation conf (I (Just (NS pt wrs)))) m
         | Just (Go d) <- fromMessage m =
-                         case sortby d $ filter (inr d pt . snd) wrs of
+                         case navigable d pt wrs of
                          []        -> return Nothing
                          ((w,r):_) -> do modify focusWindowHere
                                          return $ Just $ Left $ WindowNavigation conf $ I $ Just $
@@ -154,7 +155,7 @@ instance LayoutModifier WindowNavigation Window where
                                    has x (Just (W.Stack t l rr)) = x `elem` (t : l ++ rr)
 
         | Just (Swap d) <- fromMessage m =
-             case sortby d $ filter (inr d pt . snd) wrs of
+             case navigable d pt wrs of
              []        -> return Nothing
              ((w,_):_) -> do let swap st = unint (W.focus st) $ map (swapw (W.focus st)) $ W.integrate st
                                  swapw y x | x == w = y
@@ -170,7 +171,7 @@ instance LayoutModifier WindowNavigation Window where
                              windows $ W.modify' swap
                              return Nothing
         | Just (Move d) <- fromMessage m =
-             case sortby d $ filter (inr d pt . snd) wrs of
+             case navigable d pt wrs of
              []        -> return Nothing
              ((w,_):_) -> do mst <- gets (W.stack . W.workspace . W.current . windowset)
                              return $ do st <- mst
@@ -183,6 +184,9 @@ instance LayoutModifier WindowNavigation Window where
                handleMessOrMaybeModifyIt (WindowNavigation conf (I $ Just (NS pt wrs))) (SomeMessage Hide)
     handleMessOrMaybeModifyIt _ _ = return Nothing
 
+navigable :: Direction -> Point -> [(Window, Rectangle)] -> [(Window, Rectangle)]
+navigable d pt = sortby d . filter (inr d pt . snd)
+
 truncHead :: [a] -> [a]
 truncHead (x:_) = [x]
 truncHead [] = []
