@@ -43,6 +43,7 @@ module XMonad.Prompt
     , uniqSort
     , decodeInput
     , encodeOutput
+    , historyCompletion
     ) where
 
 import XMonad  hiding (config, io)
@@ -51,10 +52,12 @@ import XMonad.Util.Font
 import XMonad.Util.XSelection (getSelection)
 
 import Control.Arrow ((&&&))
+import Control.Concurrent (threadDelay)
 import Control.Monad.Reader
 import Control.Monad.State
 import Control.Applicative ((<$>))
 import Data.Char
+import Data.Bits ((.&.))
 import Data.Maybe
 import Data.List
 import Data.Set (fromList, toList)
@@ -74,17 +77,17 @@ type XP = StateT XPState IO
 
 data XPState =
     XPS { dpy                :: Display
-        , rootw              :: Window
-        , win                :: Window
-        , screen             :: Rectangle
+        , rootw              :: !Window
+        , win                :: !Window
+        , screen             :: !Rectangle
         , complWin           :: Maybe Window
         , complWinDim        :: Maybe ComplWindowDim
         , completionFunction :: String -> IO [String]
-        , gcon               :: GC
-        , fontS              :: XMonadFont
-        , xptype             :: XPType
+        , gcon               :: !GC
+        , fontS              :: !XMonadFont
+        , xptype             :: !XPType
         , command            :: String
-        , offset             :: Int
+        , offset             :: !Int
         , history            :: [History]
         , config             :: XPConfig
         }
@@ -96,11 +99,13 @@ data XPConfig =
         , fgHLight          :: String     -- ^ Font color of a highlighted completion entry
         , bgHLight          :: String     -- ^ Background color of a highlighted completion entry
         , borderColor       :: String     -- ^ Border color
-        , promptBorderWidth :: Dimension  -- ^ Border width
+        , promptBorderWidth :: !Dimension -- ^ Border width
         , position          :: XPPosition -- ^ Position: 'Top' or 'Bottom'
-        , height            :: Dimension  -- ^ Window height
-        , historySize       :: Int        -- ^ The number of history entries to be saved
+        , height            :: !Dimension -- ^ Window height
+        , historySize       :: !Int       -- ^ The number of history entries to be saved
         , defaultText       :: String     -- ^ The text by default in the prompt line
+        , autoComplete      :: Maybe Int  -- ^ Just x: if only one completion remains, auto-select it,
+                                          --   and delay by x microseconds
         } deriving (Show, Read)
 
 data XPType = forall p . XPrompt p => XPT p
@@ -169,6 +174,7 @@ defaultXPConfig =
         , height            = 18
         , historySize       = 256
         , defaultText       = []
+        , autoComplete      = Nothing
         }
 
 type ComplFunction = String -> IO [String]
@@ -301,6 +307,25 @@ completionHandle _ ks (KeyEvent {ev_event_type = t, ev_state = m})
 -- some other event: go back to main loop
 completionHandle _ k e = handle k e
 
+tryAutoComplete :: XP Bool
+tryAutoComplete = do
+    ac <- gets (autoComplete . config)
+    case ac of
+        Just d -> do cs <- getCompletions
+                     case cs of
+                         [c] -> runCompleted c d >> return True
+                         _   -> return False
+        Nothing    -> return False
+  where runCompleted cmd delay = do
+            st <- get
+            let new_command = nextCompletion (xptype st) (command st) [cmd]
+            modify $ \s -> s { command = "autocompleting..." }
+            updateWindows
+            io $ threadDelay delay
+            modify $ \s -> s { command = new_command }
+            historyPush
+            return True
+
 -- KeyPresses
 
 data Direction = Prev | Next deriving (Eq,Show,Read)
@@ -308,7 +333,7 @@ data Direction = Prev | Next deriving (Eq,Show,Read)
 keyPressHandle :: KeyMask -> KeyStroke -> XP ()
 -- commands: ctrl + ... todo
 keyPressHandle mask (ks,_)
-    | mask == controlMask =
+    | (mask .&. controlMask) > 0 =
         -- control sequences
         case () of
           _ | ks == xK_u               -> killBefore    >> go
@@ -320,6 +345,7 @@ keyPressHandle mask (ks,_)
             | ks == xK_Left            -> moveWord Prev >> go
             | ks == xK_Delete          -> killWord Next >> go
             | ks == xK_BackSpace       -> killWord Prev >> go
+            | ks == xK_w               -> killWord Prev >> go
             | ks == xK_g || ks == xK_c -> quit
             | otherwise  -> eventLoop handle -- unhandled control sequence
     | ks == xK_Return    = historyPush       >> return ()
@@ -340,7 +366,8 @@ keyPressHandle _ (_,s)
     | s == "" = eventLoop handle
     | otherwise = do insertString (decodeInput s)
                      updateWindows
-                     eventLoop handle
+                     completed <- tryAutoComplete
+                     unless completed $ eventLoop handle
 
 -- KeyPress and State
 
@@ -778,3 +805,15 @@ breakAtSpace s
 -- | Sort a list and remove duplicates.
 uniqSort :: Ord a => [a] -> [a]
 uniqSort = toList . fromList
+
+-- | 'historyCompletion' provides a canned completion function much like
+--   getShellCompl; you pass it to mkXPrompt, and it will make completions work
+--   from the query history stored in ~/.xmonad/history.
+historyCompletion :: ComplFunction
+historyCompletion = \x -> liftM (filter $ isInfixOf x) readHistoryIO
+
+-- We need to define this locally because there is no function with the type "XP a -> IO a", and
+-- 'getHistory' is uselessly of the type "XP [String]".
+readHistoryIO :: IO [String]
+readHistoryIO = do (hist,_) <- readHistory
+                   return $ map command_history hist
\ No newline at end of file
