@@ -45,6 +45,15 @@ import XMonad.Actions.WindowBringer
 -- > , ((modMask x .|. shiftMask, xK_g     ), windowPromptGoto  defaultXPConfig)
 -- > , ((modMask x .|. shiftMask, xK_b     ), windowPromptBring defaultXPConfig)
 --
+-- The autoComplete option is a handy complement here:
+--
+-- > , ((modMask x .|. shiftMask, xK_g     ), windowPromptGoto
+-- >                                            defaultXPConfig { autoComplete = Just 500000 } )
+--
+-- The \'500000\' is the number of microseconds to pause before sending you to
+-- your new window. This is useful so that you don't accidentally send some
+-- keystrokes to the selected client.
+--
 -- For detailed instruction on editing the key binding see
 -- "XMonad.Doc.Extending#Editing_key_bindings".
 
@@ -64,16 +73,14 @@ windowPromptBring c = doPrompt Bring c
 doPrompt :: WindowPrompt -> XPConfig -> X ()
 doPrompt t c = do
   a <- case t of
-         Goto  -> return . gotoAction  =<< windowMapWith (W.tag . fst)
-         Bring -> return . bringAction =<< windowMapWith snd
-  wm <- windowMapWith id
+         Goto  -> fmap gotoAction  windowMap
+         Bring -> fmap bringAction windowMap
+  wm <- windowMap
   mkXPrompt t c (compList wm) a
 
     where
-
       winAction a m    = flip whenJust (windows . a) . flip M.lookup m
-      gotoAction       = winAction W.greedyView
+      gotoAction       = winAction W.focusWindow
       bringAction      = winAction bringWindow
-      bringWindow w ws = W.shiftWin (W.tag . W.workspace . W.current $ ws) w ws
 
       compList m s = return . filter (isPrefixOf s) . map fst . M.toList $ m
