@@ -35,7 +35,7 @@ import XMonad.Actions.Submap
 import qualified Data.Map as M
 import Data.List (foldl', intersperse, sortBy, groupBy, nub)
 import Data.Ord (comparing)
-import Data.Maybe (catMaybes, isNothing, isJust, fromJust)
+import Data.Maybe
 import Control.Arrow (first, (&&&))
 
 import Text.ParserCombinators.ReadP
@@ -180,7 +180,44 @@ removeMouseBindings conf mouseBindingList =
 -- > <Insert>
 -- > <Break>
 -- > <Space>
--- > <F1>-<F12>
+-- > <F1>-<F24>
+-- > <KP_Space>
+-- > <KP_Tab>
+-- > <KP_Enter>
+-- > <KP_F1>
+-- > <KP_F2>
+-- > <KP_F3>
+-- > <KP_F4>
+-- > <KP_Home>
+-- > <KP_Left>
+-- > <KP_Up>
+-- > <KP_Right>
+-- > <KP_Down>
+-- > <KP_Prior>
+-- > <KP_Page_Up>
+-- > <KP_Next>
+-- > <KP_Page_Down>
+-- > <KP_End>
+-- > <KP_Begin>
+-- > <KP_Insert>
+-- > <KP_Delete>
+-- > <KP_Equal>
+-- > <KP_Multiply>
+-- > <KP_Add>
+-- > <KP_Separator>
+-- > <KP_Subtract>
+-- > <KP_Decimal>
+-- > <KP_Divide>
+-- > <KP_0>
+-- > <KP_1>
+-- > <KP_2>
+-- > <KP_3>
+-- > <KP_4>
+-- > <KP_5>
+-- > <KP_6>
+-- > <KP_7>
+-- > <KP_8>
+-- > <KP_9>
 
 mkKeymap :: XConfig l -> [(String, X ())] -> M.Map (KeyMask, KeySym) (X ())
 mkKeymap c = M.fromList . mkSubmaps . readKeymap c
@@ -205,16 +242,14 @@ op `on` f = \x y -> f x `op` f y
 --   @(KeyMask,KeySym)@ pairs.  Key sequences which fail to parse will
 --   be ignored.
 readKeymap :: XConfig l -> [(String, X())] -> [([(KeyMask,KeySym)], X())]
-readKeymap c = catMaybes . map (maybeKeys . first (readKeySequence c))
+readKeymap c = mapMaybe (maybeKeys . first (readKeySequence c))
   where maybeKeys (Nothing,_) = Nothing
         maybeKeys (Just k, act) = Just (k, act)
 
 -- | Parse a sequence of keys, returning Nothing if there is
 --   a parse failure (no parse, or ambiguous parse).
 readKeySequence :: XConfig l -> String -> Maybe [(KeyMask, KeySym)]
-readKeySequence c s = case parses s of
-    [k] -> Just k
-    _   -> Nothing
+readKeySequence c = listToMaybe . parses
   where parses = map fst . filter (null.snd) . readP_to_S (parseKeySequence c)
 
 -- | Parse a sequence of key combinations separated by spaces, e.g.
@@ -267,34 +302,71 @@ keyNames = functionKeys ++ specialKeys
 --   the associated KeySyms.
 functionKeys :: [(String, KeySym)]
 functionKeys = [ ("F" ++ show n, k)
-               | (n,k) <- zip ([1..12] :: [Int]) [xK_F1..] ]
+               | (n,k) <- zip ([1..24] :: [Int]) [xK_F1..] ]
 
 -- | A list of special key names and their corresponding KeySyms.
 specialKeys :: [(String, KeySym)]
-specialKeys = [ ("Backspace", xK_BackSpace)
-              , ("Tab"      , xK_Tab      )
-              , ("Return"   , xK_Return)
-              , ("Pause"    , xK_Pause)
+specialKeys = [ ("Backspace"  , xK_BackSpace)
+              , ("Tab"        , xK_Tab)
+              , ("Return"     , xK_Return)
+              , ("Pause"      , xK_Pause)
               , ("Scroll_lock", xK_Scroll_Lock)
-              , ("Sys_Req"  , xK_Sys_Req)
-              , ("Escape"   , xK_Escape)
-              , ("Esc"      , xK_Escape)
-              , ("Delete"   , xK_Delete)
-              , ("Home"     , xK_Home)
-              , ("Left"     , xK_Left)
-              , ("Up"       , xK_Up)
-              , ("Right"    , xK_Right)
-              , ("Down"     , xK_Down)
-              , ("L"        , xK_Left)
-              , ("U"        , xK_Up)
-              , ("R"        , xK_Right)
-              , ("D"        , xK_Down)
-              , ("Page_Up"  , xK_Page_Up)
-              , ("Page_Down", xK_Page_Down)
-              , ("End"      , xK_End)
-              , ("Insert"   , xK_Insert)
-              , ("Break"    , xK_Break)
-              , ("Space"    , xK_space)
+              , ("Sys_Req"    , xK_Sys_Req)
+              , ("Escape"     , xK_Escape)
+              , ("Esc"        , xK_Escape)
+              , ("Delete"     , xK_Delete)
+              , ("Home"       , xK_Home)
+              , ("Left"       , xK_Left)
+              , ("Up"         , xK_Up)
+              , ("Right"      , xK_Right)
+              , ("Down"       , xK_Down)
+              , ("L"          , xK_Left)
+              , ("U"          , xK_Up)
+              , ("R"          , xK_Right)
+              , ("D"          , xK_Down)
+              , ("Page_Up"    , xK_Page_Up)
+              , ("Page_Down"  , xK_Page_Down)
+              , ("End"        , xK_End)
+              , ("Insert"     , xK_Insert)
+              , ("Break"      , xK_Break)
+              , ("Space"      , xK_space)
+              , ("KP_Space"   , xK_KP_Space)
+              , ("KP_Tab"     , xK_KP_Tab)
+              , ("KP_Enter"   , xK_KP_Enter)
+              , ("KP_F1"      , xK_KP_F1)
+              , ("KP_F2"      , xK_KP_F2)
+              , ("KP_F3"      , xK_KP_F3)
+              , ("KP_F4"      , xK_KP_F4)
+              , ("KP_Home"    , xK_KP_Home)
+              , ("KP_Left"    , xK_KP_Left)
+              , ("KP_Up"      , xK_KP_Up)
+              , ("KP_Right"   , xK_KP_Right)
+              , ("KP_Down"    , xK_KP_Down)
+              , ("KP_Prior"   , xK_KP_Prior)
+              , ("KP_Page_Up" , xK_KP_Page_Up)
+              , ("KP_Next"    , xK_KP_Next)
+              , ("KP_Page_Down", xK_KP_Page_Down)
+              , ("KP_End"     , xK_KP_End)
+              , ("KP_Begin"   , xK_KP_Begin)
+              , ("KP_Insert"  , xK_KP_Insert)
+              , ("KP_Delete"  , xK_KP_Delete)
+              , ("KP_Equal"   , xK_KP_Equal)
+              , ("KP_Multiply", xK_KP_Multiply)
+              , ("KP_Add"     , xK_KP_Add)
+              , ("KP_Separator", xK_KP_Separator)
+              , ("KP_Subtract", xK_KP_Subtract)
+              , ("KP_Decimal" , xK_KP_Decimal)
+              , ("KP_Divide"  , xK_KP_Divide)
+              , ("KP_0"       , xK_KP_0)
+              , ("KP_1"       , xK_KP_1)
+              , ("KP_2"       , xK_KP_2)
+              , ("KP_3"       , xK_KP_3)
+              , ("KP_4"       , xK_KP_4)
+              , ("KP_5"       , xK_KP_5)
+              , ("KP_6"       , xK_KP_6)
+              , ("KP_7"       , xK_KP_7)
+              , ("KP_8"       , xK_KP_8)
+              , ("KP_9"       , xK_KP_9)
               ]
 
 -- | Given a configuration record and a list of (key sequence
