@@ -25,7 +25,9 @@ module XMonad.Util.Run (
                           runInTerm,
                           safeRunInTerm,
                           seconds,
-                          spawnPipe
+                          spawnPipe,
+
+                          hPutStr, hPutStrLn  -- re-export for convenience
                          ) where
 
 import System.Posix.IO
@@ -76,8 +78,7 @@ runProcessWithInputAndWait cmd args input timeout = do
         waitForProcess ph
         return ()
 
--- | Multiplies by ONE MILLION, for use with
--- 'runProcessWithInputAndWait'.
+-- | Multiplies by ONE MILLION, for functions that take microseconds.
 --
 -- Use like:
 --
